from typing import List

from workspaces.capability.helpers import CapabilitySequence
from workspaces.capability.schema_interfaces import CapabilityExecutionIF
from workspaces.capability.services.interfaces import (
    CapabilityEngineIF,
    CapabilityInfoIF,
)
from workspaces.system.schema import AbstractFile
from workspaces.workflow.services.interfaces import WorkflowServiceIF


class CapabilityEngine(CapabilityEngineIF):
    """
    Executes a prepare and run workflow step of a capability
    """

    def __init__(
        self,
        capability_info: CapabilityInfoIF,
        workflow_service: WorkflowServiceIF,
    ):
        self.capability_info = capability_info
        self.workflow_service = workflow_service

    def execute(self, execution: CapabilityExecutionIF):
        """
        Communicate with workflow service and send a request to run a workflow with given settings

        :param execution:
        """
        step_sequence = CapabilitySequence.from_str(execution.steps)
        # Grab value of current step (workflow name)
        cur_step = step_sequence[int(execution.current_step)]
        cur_step.execute(self, execution)

    def submit_workflow_request(
        self, execution_id: int, workflow_name: str, workflow_args: dict, files: List[AbstractFile]
    ):
        """
        Submit a workflow request to the workflow service

        :param execution_id:
        :param workflow_name:
        :param workflow_args:
        :param files:
        """
        workflow_request = self.workflow_service.create_workflow_request(workflow=workflow_name, argument=workflow_args)

        execution = self.capability_info.lookup_execution(execution_id)
        execution.update_execution(workflow_request['workflow_request_id'])
        self.capability_info.save_execution(execution)

        self.workflow_service.execute(workflow_request, files)

